@@ -61,15 +61,18 @@ sub make_doc {
 ############################ create pcap files
 sub make_pcaps {
     my $base = shift;
+    $NOGARBLE = 1;
     eval { require Net::PcapWriter }
 	or die "cannot load Net::PcapWriter\n";
     -d $base or die "$base does not exist";
     for my $cat ( App::DubiousHTTP::Tests->categories ) {
 	$cat->TESTS or next;
-	my $dir = "$base/".$cat->ID;
+	( my $id = $cat->ID ) =~s{[^\w\-.,;+=]+}{_}g;
+	my $dir = "$base/$id";
 	-d $dir or mkdir($dir) or die "cannot create $dir: $!";
 	for my $tst ( $cat->TESTS ) {
-	    my $pc = Net::PcapWriter->new( "$dir/".$tst->ID.".pcap" );
+	    ( my $id = $tst->ID ) =~s{[^\w\-.,;+=]+}{_}g;
+	    my $pc = Net::PcapWriter->new( "$dir/$id.pcap" );
 	    my $conn = $pc->tcp_conn('1.1.1.1',1111,'8.8.8.8',80);
 	    $conn->write(0, "GET ".$tst->url('eicar.txt')." HTTP/1.1\r\nHost: foo.bar\r\n\r\n" );
 	    $conn->write(1, $tst->make_response('eicar.txt') );
@@ -84,12 +87,16 @@ sub serve {
 	my ($path,$listen,$rqhdr,$payload) = @_;
 	return "HTTP/1.0 404 not found\r\n\r\n" if $path eq '/favicon.ico';
 
-	if ($path =~m{\A/submit_results} && $payload) {
+	if ($path =~m{\A/submit_(?:(details)|results)} && $payload) {
+	    my $details = $1;
 	    $rqhdr .= $payload;
 	    $rqhdr =~s{( /=[A-Za-z0-9_\-]+={0,2} )}{ ungarble_url($1) }eg;
 	    $rqhdr =~s{^}{ }mg;
 	    warn $rqhdr;
-	    return "HTTP/1.0 204 ok\r\n\r\n";
+	    return "HTTP/1.0 204 ok\r\n\r\n" if ! $details;
+	    return "HTTP/1.0 200 ok\r\nContent-type: text/html\r\n\r\n"
+		."<!doctype html>"
+		."<h1>Thanks for providing us with the feedback.</h1>";
 	}
 
 	local $BASE_URL = "http://$listen";
