@@ -121,6 +121,8 @@ sub auto {
     my ($hdr,$body,$isbad) = content($page);
     $html .= "<script>\n";
 
+    $html .= sprintf("reference='%x%x';\n", rand(2**32), time());
+
     my ($accept) = $rqhdr =~m{^Accept:[ \t]*([^\r\n]+)}mi;
     if ($qstring =~m{(?:^|\&)accept=([^&]*)}) {
 	($accept = $1) =~s{(?:%([a-f\d]{2})|(\+))}{ $2 ? ' ' : chr(hex($1)) }esg;
@@ -131,14 +133,28 @@ sub auto {
     $isbad ||= '';
     $html .= "isbad ='$isbad';\n";
     $html .= "var checks = [];\n";
-    $html .= "checks.push({ page:'". garble_url("/clen/$page/close,clen,content"). 
-	"', desc:'sanity check', valid:1, expect_bad:1 });\n";
+    if ($isbad) {
+	$html .= "expect64_harmless = '".encode_base64( (content('novirus.txt'))[1],'')."';\n";
+	$html .= "checks.push({ page:'". garble_url("/clen/novirus.txt/close,clen,content").
+	    "', desc:'sanity check without test virus', valid:2, log_header:1, harmless:1, file: 'novirus.txt' });\n";
+	$html .= "checks.push({ page:'". garble_url("/clen/$page/close,clen,content").
+	    "', desc:'sanity check with test virus', valid:2, expect_bad:1, log_header:1, file: '$page' });\n";
+    } else {
+	$html .= "checks.push({ page:'". garble_url("/clen/$page/close,clen,content").
+	    "', desc:'sanity check', valid:2, log_header:1, file: '$page' });\n";
+    }
     for(@cat) {
 	next if $cat ne 'all' && $_->ID ne $cat;
 	for($_->TESTS) {
-	    $html .= sprintf("checks.push({ page:'%s', desc:'%s', valid:%d });\n",
-		$_->url($page), quotemeta($_->DESCRIPTION), $_->VALID)
+	    if ($isbad) {
+		$html .= sprintf("checks.push({ page:'%s', desc:'%s', valid:%d, harmless_page: '%s', file: '%s'  });\n",
+		    $_->url($page), quotemeta(html_escape($_->DESCRIPTION)), $_->VALID, $_->url('novirus.txt'),'novirus.txt')
+	    } else {
+		$html .= sprintf("checks.push({ page:'%s', desc:'%s', valid:%d, file:'%s' });\n",
+		    $_->url($page), quotemeta(html_escape($_->DESCRIPTION)), $_->VALID,$page)
+	    }
 	}
+
     }
     $html .= "runtests(checks,0);\n</script>\n";
     return "HTTP/1.0 200 ok\r\n".
@@ -153,8 +169,12 @@ sub _auto_static_html { return <<'HTML'; }
 <meta charset="utf-8">
 <style>
 body      { font-family: Verdana, sans-serif; }
-#nobad    { padding: 1em; margin: 1em; background: red; display: none; }
+#nobad    { padding: 2em; margin: 1em; background: #ff3333; display: none; }
+#nobad div   { font-size: 150%; margin: 0.5em;  }
 #noevade  { padding: 1em; margin: 1em; background: green; display: none; }
+#evadable { padding: 1em; margin: 1em; background: #ff3333; display: none; }
+#urlblock { padding: 1em; margin: 1em; background: #ffff00  ; display: none; }
+#urlblock div  { font-size: 150%; margin: 0.5em;  }
 #notice   { padding: 1em; margin: 1em; background: #e9f2e1; display: none; }
 #warnings { padding: 1em; margin: 1em; background: #e3a79f; display: none; }
 #process  { padding: 1em; margin: 1em; background: #f2f299; }
@@ -162,12 +182,16 @@ body      { font-family: Verdana, sans-serif; }
 .desc     { font-size: 110%; }
 .srclink  { font-variant: small-caps; }
 .trylink  { font-variant: small-caps; }
+#eicar    { font-family: Lucida Sans Typewriter,Lucida Console,monaco,Bitstream Vera Sans Mono,monospace; padding: 0.5em; margin: 0.5em; border-style: solid; border-width: 1px; }
 </style>
 <div id=noscript>
 You need to have JavaScript enabled to run this tests.
 </div>
-<div id=process></div>
 <div id=nobad> </div>
+<div id=urlblock> </div>
+<div id=evasions></td>
+<div id=process></div>
+<div id=evadable> </div>
 <div id=noevade> </div>
 <div id=warnings><h1>Serious Problems</h1><ol id=ol_warnings></ol></div>
 <div id=notice><h1>Behavior in Uncommon Cases</h1><ol id=ol_notice></ol></div>
@@ -211,30 +235,45 @@ var div_ol_notice = document.getElementById('ol_notice');
 var div_warnings = document.getElementById('warnings');
 var div_ol_warnings = document.getElementById('ol_warnings');
 var div_nobad = document.getElementById('nobad');
+var div_evasions = document.getElementById('evasions');
 var div_process = document.getElementById('process');
 var expect64;
+var expect64_harmless;
 var isbad;
 var results = '';
+var reference;
 var accept = null;
 
-function add_warning(m,page,desc) {
-    div_ol_warnings.innerHTML = div_ol_warnings.innerHTML + "<li>" + m + ": <span class=desc>" + desc + "</span>" +
-	"&nbsp;<a class=trylink target=_blank href=" + page + ">try</a>" +
-	"&nbsp;<a class=srclink target=_blank href=/src" + page + ">src</a>" +
+function add_warning(m,test) {
+    div_ol_warnings.innerHTML = div_ol_warnings.innerHTML + "<li>" + m + ": <span class=desc>" + test['desc'] + "</span>" +
+	"&nbsp;<a class=trylink download='" + test['file'] + "' href=" + test['page'] + ">try</a>" +
+	"&nbsp;<a class=srclink target=_blank href=/src" + test['page'] + ">src</a>" +
 	"</li>";
     div_warnings.style.display = 'block';
 }
 
-function add_notice(m,page,desc) {
-    div_ol_notice.innerHTML = div_ol_notice.innerHTML + "<li>" + m + ": <span class=desc>" + desc + "</span>" +
-	"&nbsp;<a class=trylink target=_blank href=" + page + ">try</a>" +
-	"&nbsp;<a class=srclink target=_blank href=/src" + page + ">src</a>" +
+function add_notice(m,test) {
+    div_ol_notice.innerHTML = div_ol_notice.innerHTML + "<li>" + m + ": <span class=desc>" + test['desc'] + "</span>" +
+	"&nbsp;<a class=trylink download='" + test['file'] + "' href=" + test['page'] + ">try</a>" +
+	"&nbsp;<a class=srclink target=_blank href=/src" + test['page'] + ">src</a>" +
 	"</li>";
     div_notice.style.display = 'block';
 }
 
-function add_debug(m) {
-    div_debug.innerHTML = div_debug.innerHTML + m + "<br>";
+function add_debug(m,test) {
+    div_debug.innerHTML = div_debug.innerHTML + m + (test ?
+	"&nbsp;<a class=trylink download='" + test['file'] + "' href=" + test['page'] + ">try</a>" +
+	"&nbsp;<a class=srclink target=_blank href=/src" + test['page'] + ">src</a>" +
+	"<br>": "");
+}
+
+function escapeAttribute(attr) {
+    return attr
+         .replace(/&/g, "&amp;")
+         .replace(/</g, "&lt;")
+         .replace(/>/g, "&gt;")
+         .replace(/"/g, "&quot;")
+         .replace(/'/g, "&#039;");
 }
 
 function _log(m) {
@@ -243,6 +282,8 @@ function _log(m) {
 }
 
 var evasions = 0;
+var evasions_blocked = 0;
+var browser_invalid = 0;
 function xhr(method,page,payload,callback) {
     var req = null;
     try { req = new XMLHttpRequest(); } 
@@ -303,69 +344,125 @@ function check_page(req,test,status) {
 	if (response == null) {
 	    _log("no data for " + test['page']);
 	} else {
+	    var expect = expect64;
+	    if (test['harmless'] || test['harmless_retry']) {
+		expect = expect64_harmless;
+	    }
 	    var result64 = base64_encode(response);
-	    if (result64 == expect64) {
+	    if (result64 == expect) {
 		status = 'match';
 	    } else {
 		status = 'change(' + status + ')';
 		_log( "len=" + response.length + "   " + test['page'] + ' - ' + test['desc'] );
 		_log( "response: " + result64 );
-		_log( "expect:   " + expect64 );
+		_log( "expect:   " + expect );
 	    }
 	}
     }
-    add_debug( test['desc'] + ' - ' + status );
-    if (isbad != '') {
+
+    if (test['log_header']) {
+	// i.e. Via added or similar
+	try {
+	    var header = req.getAllResponseHeaders();
+	    results = results + "H | " + test['page'] + " | " + base64_encode(header) + "\n";
+	} catch(e) {}
+    }
+
+    add_debug( test['desc'] + ' - ' + status + ( test['harmless_retry'] ? ' - retry with harmless content':''), test);
+
+    if (test['harmless'] || test['harmless_retry']) {
+	// perfectly good response should pass, bad might fail or not
+	if (status != 'match') {
+	    browser_invalid++;
+	    if (test['valid'] == 2) { // no browser should fail on this!
+		add_warning("Failed to load harmless and perfectly valid response",test);
+		results = results + "X | " + status + " | " + test['page'] + " | " + test['desc'] + " | failed harmless\n";
+		results = results + "T | " + test['page'] + " | " + result64 + "\n";
+		var div_urlblock = document.getElementById('urlblock');
+		div_urlblock.innerHTML = "<div>" 
+		    + "The firewall blocked a harmless and perfectly valid response from the server, which did not contain any kind of evasion attempts.<br>"
+		    + "It might be that the firewall blocked the access based on URL filtering and not based on the response at all. "
+		    + "This means any results you get during this tests should be considered with great caution because they might not actually reflect "
+		    + "the abilities of the firewall to detect malware."
+		    + "</div>";
+		div_urlblock.style.display = 'block';
+	    } else if (['valid']>0) {
+		add_notice("Failed to load harmless and valid response, might be browser bug",test);
+		results = results + "X | " + status + " | " + test['page'] + " | " + test['desc'] + " | failed harmless\n";
+		results = results + "T | " + test['page'] + " | " + result64 + "\n";
+	    } else {
+		results = results + "B | " + status + " | " + test['page'] + " | " + test['desc'] + " | failed harmless\n";
+	    }
+	} else if (test['harmless_retry']) {
+	    // in this case an evasion attempt was blocked by the firewall
+	    evasions_blocked++;
+	    results = results + "Z | " + test['retry4status'] + " | " + test['retry4page'] + " | " + test['desc'] + " | evasion blocked\n";
+	}
+	return status;
+    }
+
+
+    if (isbad) {
 	// check for evasion
 	if (status == 'match') {
 	    if (test['expect_bad']) {
 		// assume no or stupid content filter
-		div_nobad.innerHTML = div_nobad.innerHTML + "No content filter detecting " + isbad + ".<br>"
-		    + "Assuming no content filter.<br>";
+		div_nobad.innerHTML = div_nobad.innerHTML + "<div>" +
+		    "It looks like no malware filtering is done by the firewall since " + isbad +
+		    " could not be detected when transferred using a valid and typical HTTP response.</div><div>" +
+		    "The tests will continue but it is assumed that there is no malware filter available. " +
+		    "This means no firewall bypasses can be detected (there is nothing to bypass) but instead " +
+		    "it will only check the behavior of the browser regarding atypical or malformed responses." +
+		    "</div><div>" +
+		    "If you feel that your firewall should be able to detect the malware please check your " +
+		    "firewall configuration and make sure that antivirus is enabled. This test uses only " + isbad +
+		    " which any antivirus product should be able to detect." +
+		    "</div>";
 		div_nobad.style.display = 'block';
 		results = results + "I | " + status + " | " + test['page'] + " | " + test['desc'] + " | no content filter\n";
 		isbad = '';
 	    } else {
 		// possible evasion of content filter
-		add_warning("Evasion possible",test['page'],test['desc']);
+		add_warning("Evasion possible",test);
 		results = results + "E | " + status + " | " + test['page'] + " | " + test['desc'] + " | evasion\n";
 		evasions++;
 	    }
 	} else if (test['expect_bad']) {
 	    // add answer to results, maybe we can get the type of firewall from the error message
-	    results = results + "T | " + result64 + "\n";
+	    results = results + "T | " + test['page'] + " | " + result64 + "\n";
 	}
-	return;
+	return status;
     }
 
     // check for standard conformance
     if (status == 'match') {
 	if (test['valid'] == 0) {
-	    add_warning("success for bad response",test['page'],test['desc']);
+	    add_warning("success for bad response",test);
 	    results = results + "W | " + status + " | " + test['page'] + " | " + test['desc'] + " | success for bad response\n";
 	} else if (test['valid'] == -1) {
-	    add_notice("success for valid uncommon response",test['page'],test['desc']);
+	    add_notice("success for valid uncommon response",test);
 	    results = results + "N | " + status + " | " + test['page'] + " | " + test['desc'] + " | success for valid uncommon response\n";
 	} else if (test['valid']<0) {
-	    add_notice("success for invalid uncommon response",test['page'],test['desc']);
+	    add_notice("success for invalid uncommon response",test);
 	    results = results + "N | " + status + " | " + test['page'] + " | " + test['desc'] + " | success for invalid uncommon response\n";
 	} else {
 	    results = results + "I | " + status + " | " + test['page'] + " | " + test['desc'] + " | ok\n";
 	}
     } else {
 	if (test['valid']>0) {
-	    add_warning("failure for valid response",test['page'],test['desc']);
+	    add_warning("failure for valid response",test);
 	    results = results + "W | " + status + " | " + test['page'] + " | " + test['desc'] + " | failure for valid response\n";
 	} else if (test['valid'] == -1) {
-	    add_notice("failure for valid uncommon response",test['page'],test['desc']);
+	    add_notice("failure for valid uncommon response",test);
 	    results = results + "N | " + status + " | " + test['page'] + " | " + test['desc'] + " | failure for valid uncommon response\n";
 	} else if (test['valid'] < 0) {
-	    add_notice("failure for invalid uncommon response",test['page'],test['desc']);
+	    add_notice("failure for invalid uncommon response",test);
 	    results = results + "N | " + status + " | " + test['page'] + " | " + test['desc'] + " | failure for invalid uncommon response\n";
 	} else {
 	    results = results + "I | " + status + " | " + test['page'] + " | " + test['desc'] + " | ok\n";
 	}
     }
+    return status;
 }
 
 var rand = Math.random();
@@ -373,24 +470,56 @@ function runtests(todo,done) {
     var test = todo.shift();
     if (test) {
 	var total = todo.length + done;
-	div_process.innerHTML = "Progress: " + done + "/" + total + " - " + test['desc'];
+	div_process.innerHTML = "Progress: " + (100*done/total).toFixed(1) + "% - " + test['desc'];
 	xhr('GET',test['page'] + '?rand=' + rand,null,function(req,status) {
-	    check_page(req,test,status);
+	    status = check_page(req,test,status);
+	    if (isbad && test['harmless_page'] && status != 'match') {
+		// malware not found, either because the firewall filtered it
+		// or because the browser did not understand the response.
+		// check for the last by trying with novirus.txt
+		todo.unshift({ page: test['harmless_page'], desc: test['desc'], valid: test['valid'], harmless_retry:1,
+		    retry4status:status, retry4page: test['page']});
+	    }
 	    runtests(todo,done+1);
 	});
     } else {
 	div_process.style.display = 'none';
 	add_debug("*DONE*");
-	if (isbad != '') {
+	if (isbad) {
+	    var div;
 	    if (evasions == 0) {
-		var div = document.getElementById('noevade');
-		div.style.display = 'block';
-		div.innerHTML = "<h1>Congratulations!<br>No evasions detected.</h1>";
 		results = results + "NO EVASIONS\n";
+		div = document.getElementById('noevade');
+		div.innerHTML = "<h1>Congratulations!<br>No evasions detected.</h1>"
+		    + evasions_blocked + " evasions attempts were blocked by the firewall and " 
+		    + browser_invalid + " attempts failed because the browser considered the response invalid or because the firewall blocks (invalid) responses even if there is no malware payload."
+		    + "Please note that these might be considered valid by other browsers and might lead to possible evasions, so better try with other browsers too."
+		    + "<br><br>To get an overview which products behave that nicely "
+		    + "it would be helpful if you provide us with information about the firewall product you use. "
+		    + "Please add as much details as you know and like to offer, i.e. model, patch level, specific configurations. ";
+	    } else {
+		div = document.getElementById('evadable');
+		div.innerHTML = "<h1>Danger!<br>Possible evasions detected!</h1>"
+		    + "The test detected that " + evasions + " evasion attempts were not blocked by the firewall.<br>"
+		    + evasions_blocked + " evasions attempts were blocked by the firewall and " 
+		    + browser_invalid + " attempts failed because the browser considered the response invalid or because the firewall blocks (invalid) responses even if there is no malware payload."
+		    + "Please note that these might be considered valid by other browsers and might lead to possible evasions, so better try with other browsers too.<br>"
+		    + "Since the test differs slightly from a manually triggered download it might be that some of the detected evasions are "
+		    + "not usable in reality, so please make sure the evasion works by clicking the [TRY] link "
+		    + "and comparing the downloaded file with the EICAR test virus. The file should be 68 byte and contain the string "
+		    + "<p><span id=eicar>X5O!P%@AP" + "[4\PZX54(P^)" + "7CC)7}$EICAR-STAND" + "ARD-ANTIVI" + "RUS-TEST-FILE!$H+H*</span></p>"
+		    + "To get an overview which products are affected by which evasions and to inform the maker of the product about the problems " 
+		    + "it would be helpful if you provide us with information about the firewall product you use. "
+		    + "Please add as much details as you know and like to offer, i.e. model, patch level, specific configurations. ";
 	    }
-	    xhr('POST','/submit_results/evasions=' + evasions ,results, null);
+	    div.innerHTML += '<br><br><form enctype="multipart/form-data" method=POST action="/submit_details/' + reference + '/evasions=' + evasions + "/evasions_blocked=" + evasions_blocked + '">'
+		+ '<input type=hidden name=results value="' + escapeAttribute(results) + '">'
+		+ '<textarea name=product cols=80 rows=4>... please add product description here ...</textarea>'
+		+ '<br><input type=submit name=Send></form>';
+	    div.style.display = 'block';
+	    xhr('POST','/submit_results/' + reference + '/evasions=' + evasions + "/evasions_blocked=" + evasions_blocked, results, null);
 	} else {
-	    xhr('POST','/submit_results' ,results, null);
+	    xhr('POST','/submit_results/' + reference ,results, null);
 	}
     }
 }
