@@ -19,29 +19,49 @@ DESC
 
     # ------------------------ Tests -----------------------------------
     [ 'VALID: basic tests' ],
-    [ VALID, 'chunked' => 'valid chunking'],
-    [ VALID, 'clen' => 'content-length not chunked'],
+    [ VALID, 'chunked' => 'simple and valid chunking'],
+    [ VALID, 'clen' => 'content-length header, not chunked'],
 
     [ 'VALID: use of extensions in chunked header' ],
-    [ VALID, 'chunked-ext-junk' => "some junk chunk extension" ],
-    [ VALID, 'chunked-ext-chunk' => "some junk chunk extension looking like a chunk" ],
+    [ VALID, 'chunked-ext-junk' => "chunked with some junk chunk extension" ],
+    [ VALID, 'chunked-ext-chunk' => "chunked with some junk chunk extension looking like a chunk" ],
 
     [ 'VALID: combined with content-length' ],
     # according to RFC2616 TE chunked has preference to clen
-    [ VALID, 'chunked,clen' => 'chunking and content-length'],
+    [ VALID, 'chunked,clen' => 'chunking and content-length, served chunked'],
     # but some still expect clen bytes
-    [ VALID, 'chunked,clen200' => 'chunking and content-length header with double length'],
-    [ VALID, 'chunked,clen50'  => 'chunking and content-length header with half length'],
+    [ VALID, 'chunked,clen200' => 'chunking and content-length header with double length, served chunked'],
+    [ VALID, 'chunked,clen50'  => 'chunking and content-length header with half length, served chunked'],
 
     [ 'INVALID: chunking is only allowed with HTTP/1.1' ],
     [ INVALID, 'chunked,http10' => 'Chunked Header and HTTP/1.0. Served chunked.'],
     [ INVALID, 'chunked,clen,http10' => 'Chunked Header and Content-length and HTTP/1.0. Served chunked.'],
+    [ 'INVALID: chunking is only allowed with HTTP/1.1' ],
+
+    [ 'INVALID: chunking with invalid HTTP versions' ],
+    [ INVALID, 'chunked,HTTP/1.2' => 'Chunked Header and HTTP/1.2. Served chunked.'],
+    [ INVALID, 'chunked,clen,HTTP/1.2,do_clen' => 'Chunked Header and HTTP/1.2. Not served chunked.'],
+    [ INVALID, 'chunked,HTTP/2.0' => 'Chunked Header and HTTP/2.0. Served chunked.'],
+    [ INVALID, 'chunked,clen,HTTP/2.0,do_clen' => 'Chunked Header and HTTP/2.0. erved chunked.'],
+    [ INVALID, 'chunked,HTTP/2.1' => 'Chunked Header and HTTP/2.1. Served chunked.'],
+    [ INVALID, 'chunked,clen,HTTP/2.1,do_clen' => 'Chunked Header and HTTP/2.1. erved chunked.'],
+    [ INVALID, 'chunked,HTTP/0.9' => 'Chunked Header and HTTP/0.9. Served chunked.'],
+    [ INVALID, 'chunked,clen,HTTP/0.9,do_clen' => 'Chunked Header and HTTP/0.9. Not served chunked.'],
+    [ INVALID, 'chunked,HTTP/1.01' => 'Chunked Header and HTTP/1.01. Served chunked.'],
+    [ INVALID, 'chunked,clen,HTTP/1.01,do_clen' => 'Chunked Header and HTTP/1.01. Not served chunked.'],
+    [ INVALID, 'chunked,HTTP/1.10' => 'Chunked Header and HTTP/1.10. Served chunked.'],
+    [ INVALID, 'chunked,clen,HTTP/1.10,do_clen' => 'Chunked Header and HTTP/1.10. Not served chunked.'],
+    [ INVALID, 'chunked,http/1.1' => 'Chunked Header and http/1.1. Served chunked.'],
+    [ INVALID, 'chunked,clen,http/1.1,do_clen' => 'Chunked Header and http/1.1. Not served chunked.'],
+    [ INVALID, 'chunked,http/1.0' => 'Chunked Header and http/1.0. Served chunked.'],
+    [ INVALID, 'chunked,clen,http/1.0,do_clen' => 'Chunked Header and http/1.0. Not served chunked.'],
 
     [ 'VALID: valid variations on "chunked" value' ],
-    [ VALID, 'chUnked' => 'valid chunking mixed case'],
-    [ VALID, 'rfc2047,do_clen' => "rfc2047 encoded with base64, not served chunked" ],
-    [ VALID, 'rfc2047,clen,do_clen' => "rfc2047 encoded with base64, serve with content-length" ],
-    [ UNCOMMON_VALID,'nl-chunked' => "chunked header with continuation line. Served chunked."],
+    [ VALID, 'chUnked' => 'mixed case "chUnked", served chunked'],
+    [ VALID, 'rfc2047,do_clen' => 'rfc2047/base64 encoded "chunked", not served chunked' ],
+    [ VALID, 'rfc2047,clen,do_clen' => 'rfc2047/base64 encoded "chunked" and content-length, not served chunked' ],
+    [ UNCOMMON_VALID,'nl-chunked' => "chunked header with continuation line, served chunked"],
+    [ UNCOMMON_VALID,'nl-nl-chunked' => "chunked header with double continuation line, served chunked"],
 
     [ 'INVALID: invalid variations on "chunked" value' ],
     [ INVALID, 'chu' => '"chu" not "chunked"'],
@@ -51,17 +71,17 @@ DESC
     [ INVALID, 'chunked-x' => '"chunked x" not "chunked"'],
     [ INVALID, 'x-chunked' => '"x chunked" not "chunked"'],
     [ INVALID, 'x-nl-chunked' => '"x-folding-chunked" not "chunked"'],
-    [ INVALID, 'rfc2047,do_chunked' => "rfc2047 encoded with base64, serve chunked" ],
+    [ INVALID, 'rfc2047,do_chunked' => 'rfc2047/base64 encoded "chunked", served chunked' ],
     [ INVALID, 'xte,chunked,do_chunked' => "double Transfer-Encoding: first junk, last chunked. Served chunked." ],
     [ INVALID, 'chunked,xte,do_chunked' => "double Transfer-Encoding: first chunked, last junk. Served chunked." ],
     [ INVALID, 'chunked,xte,clen,do_chunked' => "double Transfer-Encoding: first chunked, last junk. Also Content-length header. Served chunked." ],
     [ INVALID, 'xte,chunked,clen,do_chunked' => "double Transfer-Encoding: first junk, last chunked. Also Content-length header. Served chunked." ],
+    [ INVALID, 'chunked,clen,do_clen' => 'chunking and content-length, not served chunked'],
     [ INVALID,'nl-chunked,do_clen' => "chunked header with continuation line. Not served chunked."],
+    [ INVALID,'ce-chunked,do_chunked' => "Content-encoding chunked instead of Transfer-encoding. Served chunked."],
 
     [ 'INVALID: invalid chunks' ],
     [ INVALID, 'chunked-lf' => "chunk with LF as delimiter instead of CRLF" ],
-
-
 );
 
 
@@ -69,7 +89,7 @@ sub make_response {
     my ($self,$page,$spec) = @_;
     return make_index_page() if $page eq '';
     my ($hdr,$data) = content($page,$spec) or die "unknown page $page";
-    my $version = '1.1';
+    my $version = 'HTTP/1.1';
     my $te;
     for (split(',',$spec)) {
 	if ( ! $_ || $_ eq 'chunked' ) {
@@ -79,11 +99,15 @@ sub make_response {
 	    $te = $_
 	} elsif ( $_ eq 'chUnked' ) {
 	    $hdr .= "Transfer-Encoding: chUnked\r\n"
-	} elsif ( m{^(.*-)?nl-chunked$} ) {
+	} elsif ( m{^(.*-)?nl-(nl-)?chunked$} ) {
 	    my $prefix = $1 ||'';
-	    $hdr .= "Transfer-Encoding: $prefix\r\n chunked\r\n"
+	    $hdr .= "Transfer-Encoding: $prefix\r\n ";
+	    $hdr .= "\r\n " if $2;
+	    $hdr .= "chunked\r\n"
 	} elsif ( $_ eq 'chu' ) {
 	    $hdr .= "Transfer-Encoding: chu\r\n"
+	} elsif ( $_ eq 'ce-chunked' ) {
+	    $hdr .= "Content-Encoding: chunked\r\n"
 	} elsif ( $_ eq 'xchunked' ) {
 	    $hdr .= "Transfer-Encoding: xchunked\r\n"
 	} elsif ( $_ eq 'chunkedx' ) {
@@ -95,7 +119,9 @@ sub make_response {
 	} elsif ( $_ =~ m{^clen(\d+)?$} ) {
 	    $hdr .= "Content-length: ". int(($1||100)/100*length($data)) ."\r\n"
 	} elsif ( $_ eq 'http10' ) {
-	    $version = "1.0";
+	    $version = "HTTP/1.0";
+	} elsif ( $_ =~m{^HTTP/\S+}i ) {
+	    $version = $_;
 	} elsif ( $_ eq 'do_clen' ) {
 	    $te = 'clen'
 	} elsif ( $_ eq 'do_chunked' ) {
@@ -113,7 +139,7 @@ sub make_response {
 	    die $_
 	}
     }
-    $hdr = "HTTP/$version 200 ok\r\n$hdr";
+    $hdr = "$version 200 ok\r\n$hdr";
     $te ||= $hdr =~m{^Transfer-Encoding:}im ? 'chunked':'clen';
     if ( $te eq 'chunked' ) {
 	$data = join("", 
