@@ -14,21 +14,21 @@ DESC
 
     # ------------------------ Tests -----------------------------------
     [ 'VALID: single or no content-length' ],
-    [ VALID, 'close,clen,content' => 'single content-length with connection close'],
+    [ SANITY_VALID, 'close,clen,content' => 'single content-length with connection close'],
     #[ VALID, 'keep-alive,clen,content' => 'single content-length with keep-alive'],
     [ VALID, 'close,content' => 'no content-length with connection close'],
-    [ VALID, 'close,clen,content,junk' => 'single content-length with connection close, content followed by junk'],
-    [ VALID, 'close,clen,clen,content,junk' => 'correct content-length twice, content followed by junk' ],
+    [ VALID, 'close,clen,content,junk' => 'single content-length, content followed by junk, then connection close'],
+    [ VALID, 'close,clen,clen,content,junk' => 'correct content-length twice, content followed by junk, then connection close'],
 
     [ 'INVALID: content-length does not match content' ],
-    [ INVALID, 'close,clen200,content' => 'content-length double real content, eof after real content' ],
-    [ INVALID, 'close,clen50,content' => 'content-length half real content, eof after real content' ],
+    [ INVALID, 'close,clen200,content' => 'content-length double real content, close after real content' ],
+    [ INVALID, 'close,clen50,content' => 'content-length half real content, close after real content' ],
 
     [ 'INVALID: multiple conflicting content-length' ],
     [ INVALID, 'close,clen50,clen,content' => 'content-length half and full' ],
     [ INVALID, 'close,clen,clen50,content' => 'content-length full and half' ],
-    [ INVALID, 'close,clen200,clen,content,junk' => 'content-length double and full' ],
-    [ INVALID, 'close,clen,clen200,content,junk' => 'content-length full and double' ],
+    [ INVALID, 'close,clen200,clen,content,junk' => 'content-length double and full, content followed by junk and close' ],
+    [ INVALID, 'close,clen,clen200,content,junk' => 'content-length full and double, content followed by junk and close' ],
     [ INVALID, 'close,clen-folding100,clen200,content,junk' => 'content-length full (folded) and double' ],
     [ INVALID, 'close,xte,clen50,clen,content' => 'content-length half and full, invalid Transfer-Encoding' ],
     [ INVALID, 'close,xte,clen,clen50,content' => 'content-length full and half, invalid Transfer-Encoding' ],
@@ -37,16 +37,16 @@ DESC
     [ INVALID, 'close,xte,clen-folding100,clen200,content,junk' => 'content-length full (folded) and double, invalid Transfer-Encoding' ],
 
     [ 'INVALID: content-length header containing two numbers' ],
-    [ INVALID, 'close,clen50-folding100,content' => 'content-length half but full after line folding, eof after real content' ],
-    [ INVALID, 'close,clen50-100,content' => 'content-length half and full on same line, eof after real content' ],
-    [ INVALID, 'close,clen50-(100),content' => 'content-length half and full on same line, but full as MIME comment, eof after real content' ],
-    [ INVALID, 'close,clen100-folding50,content' => 'content-length full but half after line folding, eof after real content' ],
-    [ INVALID, 'close,clen100-50,content' => 'content-length full and half on same line, eof after real content' ],
-    [ INVALID, 'close,clen(100)-50,content' => 'content-length full and half on same line, but full as MIME comment, eof after real content' ],
-    [ INVALID, 'close,clen100-folding200,content,junk' => 'content-length full but double after line folding, eof after real content+junk' ],
-    [ INVALID, 'close,clen100-(200),content,junk' => 'content-length full and double on same line, but double as MIME comment, eof after real content+junk' ],
-    [ INVALID, 'close,clen200-folding100,content,junk' => 'content-length double but full after line folding, eof after real content+junk' ],
-    [ INVALID, 'close,clen(200)-100,content,junk' => 'content-length double and full on same line, but double as MIME comment, eof after real content+junk' ],
+    [ INVALID, 'close,clen50-folding100,content' => 'content-length half but full after line folding, close after real content' ],
+    [ INVALID, 'close,clen50-100,content' => 'content-length half and full on same line, close after real content' ],
+    [ INVALID, 'close,clen50-(100),content' => 'content-length half and full on same line, but full as MIME comment, close after real content' ],
+    [ INVALID, 'close,clen100-folding50,content' => 'content-length full but half after line folding, close after real content' ],
+    [ INVALID, 'close,clen100-50,content' => 'content-length full and half on same line, close after real content' ],
+    [ INVALID, 'close,clen(100)-50,content' => 'content-length full and half on same line, but full as MIME comment, close after real content' ],
+    [ INVALID, 'close,clen100-folding200,content,junk' => 'content-length full but double after line folding, close after real content+junk' ],
+    [ INVALID, 'close,clen100-(200),content,junk' => 'content-length full and double on same line, but double as MIME comment, close after real content+junk' ],
+    [ INVALID, 'close,clen200-folding100,content,junk' => 'content-length double but full after line folding, close after real content+junk' ],
+    [ INVALID, 'close,clen(200)-100,content,junk' => 'content-length double and full on same line, but double as MIME comment, close after real content+junk' ],
 );
 
 
